@@ -1,6 +1,6 @@
 /***********************************************************************
  *
- * Copyright (C) 2009 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2009, 2014, 2015 Graeme Gott <graeme@gottcode.org>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 
 #include <algorithm>
 #include <ctime>
+#include <random>
 
 /*****************************************************************************/
 
@@ -77,7 +78,12 @@ void Puzzle::generate() {
 	m_done = false;
 
 	// Set random seed
-	srand(time(0));
+#ifndef Q_OS_WIN
+	std::random_device rd;
+	std::mt19937 gen(rd());
+#else
+	std::mt19937 gen(time(0));
+#endif
 
 	// Add board background
 	createBackground();
@@ -89,7 +95,7 @@ void Puzzle::generate() {
 
 	// Create first piece
 	QList<int> colors = m_pieces.first()->colors();
-	std::random_shuffle(colors.begin(), colors.end());
+	std::shuffle(colors.begin(), colors.end(), gen);
 	for (int i = 0; i < 6; ++i) {
 		int color = colors.at(i);
 		m_pieces.first()->setConnector(i, color);
@@ -105,7 +111,8 @@ void Puzzle::generate() {
 		QList<int> colors2 = m_pieces.at(next)->colors();
 		colors = colors1 + colors2;
 		colors.erase(std::set_intersection(colors1.begin(), colors1.end(), colors2.begin(), colors2.end(), colors.begin()), colors.end());
-		int color = colors.at(rand() % colors.count());
+		std::uniform_int_distribution<int> dis(0, colors.count() - 1);
+		int color = colors.at(dis(gen));
 		m_pieces.at(i)->setConnector(offset, color);
 		m_pieces.at(next)->setConnector(match(offset), color);
 	}
@@ -114,7 +121,7 @@ void Puzzle::generate() {
 	for (int i = 1; i < 7; ++i) {
 		Piece* piece = m_pieces.at(i);
 		colors = piece->colors();
-		std::random_shuffle(colors.begin(), colors.end());
+		std::shuffle(colors.begin(), colors.end(), gen);
 		for (int j = 0; j < 6; ++j) {
 			if (piece->connector(j) == -1) {
 				piece->setConnector(j, colors.takeFirst());
@@ -123,9 +130,10 @@ void Puzzle::generate() {
 	}
 
 	// Randomize board
-	std::random_shuffle(m_pieces.begin(), m_pieces.end());
-	foreach (Piece* piece, m_pieces) {
-		piece->spin();
+	std::shuffle(m_pieces.begin(), m_pieces.end(), gen);
+	std::uniform_int_distribution<int> dis(0, 5);
+	for (Piece* piece : m_pieces) {
+		piece->spin(dis(gen));
 	}
 
 	// Position pieces
@@ -183,7 +191,11 @@ void Puzzle::swapPieces(Piece* piece1, Piece* piece2) {
 	int pos1 = m_pieces.indexOf(piece1);
 	int pos2 = m_pieces.indexOf(piece2);
 	Q_ASSERT(pos1 != -1 && pos2 != -1);
+#if (QT_VERSION >= QT_VERSION_CHECK(5,13,0))
+	m_pieces.swapItemsAt(pos1, pos2);
+#else
 	m_pieces.swap(pos1, pos2);
+#endif
 }
 
 /*****************************************************************************/
@@ -224,7 +236,7 @@ void Puzzle::save() {
 
 void Puzzle::createBackground() {
 	QGraphicsItem* background = addEllipse(10, 10, 300, 300, QPen(Qt::darkGray, 2), Qt::gray);
-	foreach (const QPointF& position, positions) {
+	for (const QPointF& position : positions) {
 		QGraphicsItem* item = addEllipse(0, 0, 70, 70, Qt::NoPen, Qt::darkGray);
 		item->setPos(position + QPointF(15,15));
 		item->setParentItem(background);
